unit TestQuirc1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls;

type

  { TTestLibQuircForm }

  TTestLibQuircForm = class(TForm)
    btnDecode: TButton;
    btnOpenFile: TButton;
    gbDecode: TGroupBox;
    Image1: TImage;
    mDecText: TMemo;
    procedure btnDecodeClick(Sender: TObject);
    procedure btnOpenFileClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { private declarations }
    procedure LoadImage(const FileName: string);
  public
    { public declarations }
  end;

var
  TestLibQuircForm: TTestLibQuircForm;

implementation

uses
  quirc,
  quircidentify, quircdecode;

{$R *.lfm}

{ TTestLibQuircForm }

procedure TTestLibQuircForm.FormShow(Sender: TObject);
begin
  if FileExists('test.jpg') then
    LoadImage('test.jpg');
end;

procedure TTestLibQuircForm.LoadImage(const FileName: string);
var bitmap: TBitmap;
begin
  Image1.Picture.LoadFromFile(FileName);
  bitmap:=TBitmap.Create;
  bitmap.PixelFormat := pf24bit;
  bitmap.Width := Image1.Picture.Bitmap.Width;
  bitmap.Height := Image1.Picture.Bitmap.Height;
  bitmap.Canvas.Draw(0, 0, Image1.Picture.Bitmap);
  Image1.Picture.Bitmap.Assign(bitmap);
  bitmap.Free;
end;

procedure TTestLibQuircForm.btnOpenFileClick(Sender: TObject);
var dlg: TOpenDialog;
begin
  dlg := TOpenDialog.Create(Self);
  dlg.Filter:='Image files (*.bmp,*.jpeg,*.jpg,*.gif,*.png)|*.BMP;*.JPEG;*.JPG;*.GIF;*.PNG';;
  if dlg.Execute then
    LoadImage(dlg.FileName);
  dlg.Free;
end;

procedure TTestLibQuircForm.btnDecodeClick(Sender: TObject);
var
  qreader: pquirc;
  qcode: quirc_code;
  qdata: quirc_data;
  i: integer;
  s: string;

  procedure BitmapToQuircImage(dst: PByte);
  var bitmap: TBitmap; y,x: integer; c4: DWord; src: PByte;
  begin
    bitmap:=TBitmap.Create;
    try
      // Use 24bit bitmap to match with Quirc image
      bitmap.PixelFormat := pf24bit;
      bitmap.Width := Image1.Picture.Bitmap.Width;
      bitmap.Height := Image1.Picture.Bitmap.Height;
      bitmap.Canvas.Draw(0, 0, Image1.Picture.Bitmap);

      for y := 0 to bitmap.Height - 1 do begin
        src := bitmap.ScanLine[y];
        for x := 0 to bitmap.Width - 1 do begin
          c4 := PDWord(src)^ and $FFFFFF;
          // 77(R) + 151(G) + 28(B) = 256
          dst^ := ((c4 shr 16)*77 + ((c4 shr 8) and $FF)*151 + (c4 and $FF)*28) shr 8;
          Inc(src,3);
          Inc(dst);
        end;
      end;
    finally
      bitmap.Free;
    end;
  end;

begin
  mDecText.Lines.Clear;

  qreader := quirc_new();
  if quirc_resize(qreader, Image1.Width, Image1.Height) = 0 then begin
    quirc_begin(qreader, nil, nil);
    BitmapToQuircImage(qreader^.image);
    quirc_end(qreader);
    for i:=0 to quirc_count(qreader)-1 do begin
      quirc_extract(qreader, i, qcode);
      // draw rectangle around found detected QR code
      with Image1 do begin
        Canvas.Pen.Color:=clRed;
        Canvas.Pen.Width:=1;
        Canvas.Pen.Style:=psSolid;
        Canvas.Polyline([Point(qcode.corners[0].x,qcode.corners[0].y),
                         Point(qcode.corners[1].x,qcode.corners[1].y),
                         Point(qcode.corners[2].x,qcode.corners[2].y),
                         Point(qcode.corners[3].x,qcode.corners[3].y),
                         Point(qcode.corners[0].x,qcode.corners[0].y)]);
      end;
      // decode QR code
      if quirc_decode(@qcode, qdata) = QUIRC_SUCCESS then begin
        SetString(s, @qdata.payload, qdata.payload_len);
        mDecText.Lines.Append(Format('[%d]: %s', [i,s]));
      end;
    end;
  end;
  quirc_destroy(qreader);
end;

end.

